/*******************************************************************************
 * Copyright (c) 2015-2016, IncQuery Labs Ltd. and Ericsson AB
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Akos Horvath, Abel Hegedus, Zoltan Ujhelyi, Daniel Segesdi, Tamas Borbas, Denes Harmath - initial API and implementation
 *******************************************************************************/
package com.incquerylabs.emdw.umlintegration.rules

import com.incquerylabs.emdw.umlintegration.queries.ChildEntryPointMatch
import com.incquerylabs.emdw.umlintegration.queries.EntryPointMatch
import com.incquerylabs.emdw.umlintegration.queries.ToplevelEntryPointMatch
import org.eclipse.papyrusrt.xtumlrt.common.EntryPoint
import org.eclipse.papyrusrt.xtumlrt.common.State
import org.eclipse.papyrusrt.xtumlrt.common.StateMachine
import java.util.Set
import org.eclipse.incquery.runtime.api.IncQueryEngine
import org.eclipse.uml2.uml.Pseudostate

class EntryPointRules{
	static def Set<AbstractMapping<?>> getRules(IncQueryEngine engine) {
		#{
			new EntryPointMapping(engine),
			new ToplevelEntryPointMapping(engine),
			new ChildEntryPointMapping(engine)
		}
	}
}

/**
 * Transforms Pseudostates of entry point kind to EntryPoints.
 */
class EntryPointMapping extends AbstractObjectMapping<EntryPointMatch, Pseudostate, EntryPoint> {

	new(IncQueryEngine engine) {
		super(engine)
	}
	
	override getXtumlrtClass() {
		EntryPoint
	}

	override getRulePriority() {
		CommonPriorities.VERTEX_MAPPING_PRIORITY
	}

	override getQuerySpecification() {
		entryPoint
	}

	override getUmlObject(EntryPointMatch match) {
		match.pseudostate
	}

	override createXtumlrtObject() {
		commonFactory.createEntryPoint
	}

	override updateXtumlrtObject(EntryPoint xtumlrtObject, EntryPointMatch match) {
	}

	override protected insertXtumlrtObject(EntryPoint xtumlrtObject, EntryPointMatch match) {
	}
	
}

/**
 * Inserts EntryPoints to their parent StateMachine.
 */
class ToplevelEntryPointMapping extends AbstractContainmentMapping<ToplevelEntryPointMatch, StateMachine, EntryPoint> {

	new(IncQueryEngine engine) {
		super(engine)
	}
	
	override getRulePriority() {
		Math.max(StateMachineMapping.PRIORITY, CommonPriorities.VERTEX_MAPPING_PRIORITY) + 1
	}

	override getQuerySpecification() {
		toplevelEntryPoint
	}

	override findParent(ToplevelEntryPointMatch match) {
		match.stateMachine.findXtumlrtObject(StateMachine)
	}
	
	override findChild(ToplevelEntryPointMatch match) {
		match.pseudostate.findXtumlrtObject(EntryPoint)
	}
	
	override insertChild(StateMachine parent, EntryPoint child, ToplevelEntryPointMatch match) {
		parent.top.entryPoints += child
	}

}

/**
 * Inserts EntryPoints to their parent CompositeState.
 */
class ChildEntryPointMapping extends AbstractContainmentMapping<ChildEntryPointMatch, State, EntryPoint> {

	new(IncQueryEngine engine) {
		super(engine)
	}
	
	override getRulePriority() {
		CommonPriorities.VERTEX_MAPPING_PRIORITY + 1
	}

	override getQuerySpecification() {
		childEntryPoint
	}

	override findParent(ChildEntryPointMatch match) {
		match.state.findXtumlrtObject(State)
	}
	
	override findChild(ChildEntryPointMatch match) {
		match.pseudostate.findXtumlrtObject(EntryPoint)
	}
	
	override insertChild(State parent, EntryPoint child, ChildEntryPointMatch match) {
		parent.entryPoints += child
	}

}